from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django import forms

from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.apps.auth.models import JumboUser

admin.site.unregister(User)


class JumboUserChangeForm(forms.ModelForm):
    """
    A form that creates a user, with no privileges, from the given username and password.
    """
    def __init__(self, *args, **kwargs):
        super(JumboUserChangeForm, self).__init__(*args, **kwargs)
        self.fields['username'] = forms.RegexField(label=_("Username"), max_length=30, regex=r'^[\w.@+-]+$',
                                                   help_text = _("Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only."),
                                                   error_messages = {'invalid': _("This value may contain only letters, numbers and @/./+/-/_ characters.")})
        #self.fields['password'] = forms.CharField(label=_("Password"), widget=forms.PasswordInput)

        self.fields['first_name'] = forms.CharField(label=_("First Name"))
        self.fields['last_name'] = forms.CharField(label=_("Last Name"))
        self.fields['email'] = forms.EmailField(label=_("E-mail"), max_length=75)

        #### ORGANIZATION ####
        if "organization" in settings.INSTALLED_APPS:
            from organization.models import Organization, Contact, ContactRole
            self.fields['contact_id'] = forms.CharField(required=False, label="", widget=forms.HiddenInput())
            self.fields['organization_name'] = forms.ModelChoiceField(Organization.objects.all(), label=_("Name"), required=False)
            self.fields['organization_role'] = forms.ModelChoiceField(ContactRole.objects.all(), label=_("Role"), required=False)
            if self.instance:
                try:
                    contact = Contact.objects.get(user=self.instance)
                    self.fields['contact_id'].initial = contact.id
                    if contact.organization:
                        self.fields['organization_name'].initial = contact.organization.id
                    if contact.role:
                        self.fields['organization_role'].initial = contact.role.id
                except Contact.DoesNotExist:
                    pass

    def save(self, commit=True):
        user = super(JumboUserChangeForm, self).save(commit=False)
        if "organization" in settings.INSTALLED_APPS:
            from organization.models import Organization, Contact, ContactRole
            contact = None
            if self.cleaned_data['contact_id']:
                contact = Contact.objects.get(id=self.cleaned_data['contact_id'])
            else:
                # Creo un nuovo contatto solo se e' stata selezionata l'anagrafica
                if self.cleaned_data['organization_name']:
                    contact = Contact()

            if contact:
                contact.first_name = self.cleaned_data['first_name']
                contact.last_name = self.cleaned_data['last_name']
                contact.organization = self.cleaned_data['organization_name']
                contact.role = self.cleaned_data['organization_role']
                contact.user = user
                contact.save()

        if commit:
            user.save()
        return user

    class Meta:
        model = JumboUser


class JumboUserAdmin(UserAdmin, ExtendibleModelAdmin):
    list_display = ['username', 'email', 'first_name', 'last_name', 'is_staff', 'is_superuser',
                    'get_edit_icon']

    if "ticket" in settings.INSTALLED_APPS:
        list_display.insert(6, 'get_ticket_user')

    if "document" in settings.INSTALLED_APPS:
        list_display.insert(6, 'get_document_user')

    form = JumboUserChangeForm
    contact_fieldsets = (
        (None, {'fields': ('username', 'password')}),
        (_('Personal info'), {'fields': (('first_name', 'last_name'), 'email')}),
        (_('Organization info'), {'fields': (('organization_name', 'organization_role', 'contact_id'),)}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser', 'user_permissions'), 'classes': ('collapse',)}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined'), 'classes': ('collapse',)}),
        (_('Groups'), {'fields': ('groups',), 'classes': ('collapse',)}),
    )


    def get_fieldsets(self, request, obj=None):
        if "organization" in settings.INSTALLED_APPS:
            if not obj:
                return self.add_fieldsets
            else:
                return self.contact_fieldsets
            return super(UserAdmin, self).get_fieldsets(request, obj)
        else:
            return super(JumboUserAdmin, self).get_fieldsets(request=request, obj=obj)

    def get_document_user(self, obj):
        icona = None
        icona_add_edit = None
        if "document" in settings.INSTALLED_APPS:
            if obj.is_document_user():
                icona = '<img src="/media/img/admin/icon-yes.gif" alt="True">'
                icona_add_edit = """<a href="%s?_popup=1&nobuttons=1" class="iframe">
                                        <img src="%sadmin/images/edit.gif" alt="Change Document User" title="Change Document User">
                                    </a>""" % (reverse('admin:document_documentuser_change', obj.document_user.id), settings.MEDIA_URL)
            else:
                icona = '<img src="/media/img/admin/icon-no.gif" alt="False">'
                icona_add_edit = """<a href="%s?_popup=1&nobuttons=1&user=%s" class="iframe">
                                        <img src="%sadmin/images/add_yellow.png" alt="Add Document User" title="Add Document User">
                                    </a>""" % (reverse('admin:document_documentuser_add'), obj.id, settings.MEDIA_URL)
        return "%s  %s" % (icona, icona_add_edit)
    get_document_user.allow_tags = True
    get_document_user.short_description = _("Is Document User")

    def get_ticket_user(self, obj):
        icona = None
        icona_add_edit = None
        if "ticket" in settings.INSTALLED_APPS:
            if obj.is_ticket_user():
                icona = '<img src="/media/img/admin/icon-yes.gif" alt="True">'
                icona_add_edit = """<a href="%s?_popup=1&nobuttons=1" class="iframe">
                                        <img src="%sadmin/images/edit.gif" alt="Change Ticket User" title="Change Ticket User">
                                    </a>""" % (reverse('admin:ticket_ticketuser_change', obj.ticket_user.id), settings.MEDIA_URL)
            else:
                icona = '<img src="/media/img/admin/icon-no.gif" alt="False">'
                icona_add_edit = """<a href="%s?_popup=1&nobuttons=1&user=%s" class="iframe">
                                        <img src="%sadmin/images/add_yellow.png" alt="Add Ticket User" title="Add Ticket User">
                                    </a>""" % (reverse('admin:ticket_ticketuser_add'), obj.id, settings.MEDIA_URL)
        return "%s  %s" % (icona, icona_add_edit)
    get_ticket_user.allow_tags = True
    get_ticket_user.short_description = _("Is Ticket User")
admin.site.register(JumboUser, JumboUserAdmin)