from django import forms
from django.utils.encoding import StrAndUnicode, force_unicode
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete.widgets import AutocompleteMultipleWidget
from jumbo.apps.autocomplete.widgets import AutocompleteWidget
from jumbo.apps.autocomplete.widgets import AutocompleteInlineCharFieldWidget
from jumbo.apps.autocomplete.widgets import AutocompleteInlineGFKCharFieldWidget
from jumbo.middleware.thread_local import get_request


class AutocompleteInlineCharField(forms.Field):
    widget = AutocompleteInlineCharFieldWidget

    def __init__(self, ac_name, view=autocomplete, **kwargs):
        self.ac_name = ac_name
        self.view = view
        widget = self.widget(ac_name=ac_name, view=view)
        forms.Field.__init__(self, widget=widget, **kwargs)

    def to_python(self, value):
        model, fields, limit, key, label, auth, views, force_selection = self.view.settings[self.ac_name]

        if not model:
            return value
        try:
            #model = qs.model
            p = model.objects.get(pk=value)
        except:
            raise ValidationError(_("Select a valid %(model)s") % {'model': model._meta.verbose_name})
        return p

class AutocompleteInlineGFKCharField(AutocompleteInlineCharField):
    widget = AutocompleteInlineGFKCharFieldWidget

class AutocompleteChoiceField(forms.ChoiceField):
    """
    A ModelChoiceField which uses an autocomplete widget, instead of an html
    select tag, to render its choices.
    """
    widget = AutocompleteMultipleWidget


    def __init__(self, ac_name, facebook_style=False, view=autocomplete,
                 min_chars = 1, hint_text = None, no_result_text = None, 
                 searching_text = None, on_result=None, on_select_token=None,
                 on_delete_token=None, force_selection=True, **kwargs):
        self.ac_name = ac_name
        self.view = view
        widget = self.widget(ac_name=ac_name, force_selection=force_selection,
                             view=view, facebook_style=facebook_style, min_chars=min_chars, 
                             token_limit=1, hint_text=hint_text, no_result_text=no_result_text,
                             searching_text=searching_text, on_result=on_result,
                             on_select_token=on_select_token, on_delete_token=on_delete_token)
        forms.Field.__init__(self, widget=widget, **kwargs)


    def validate(self, value):
        """
        Only validates if required
        """
        if self.required and not value:
            raise ValidationError(self.error_messages['required'])
        return value


class AutocompleteMultiChoiceField(forms.MultipleChoiceField):
    """
    A ModelChoiceField which uses an autocomplete widget, instead of an html
    select tag, to render its choices.
    """
    widget = AutocompleteMultipleWidget


    def __init__(self, ac_name, facebook_style=False, view=autocomplete,
                 min_chars = 1, token_limit = None, hint_text = None, no_result_text = None, 
                 searching_text = None, on_result=None, on_select_token=None,
                 on_delete_token=None, force_selection=True, **kwargs):
        self.ac_name = ac_name
        self.view = view
        widget = self.widget(ac_name=ac_name, force_selection=force_selection,
                             view=view, facebook_style=facebook_style, min_chars=min_chars, 
                             token_limit=token_limit, hint_text=hint_text, no_result_text=no_result_text,
                             searching_text=searching_text, on_result=on_result,
                             on_select_token=on_select_token, on_delete_token=on_delete_token)
        forms.Field.__init__(self, widget=widget, **kwargs)


    def validate(self, value):
        """
        Only validates if required
        """
        if self.required and not value:
            raise ValidationError(self.error_messages['required'])
        return value


class AutocompleteModelMultiChoiceField(forms.ModelMultipleChoiceField):
    """
    A ModelChoiceField which uses an autocomplete widget, instead of an html
    select tag, to render its choices.
    """
    widget = AutocompleteMultipleWidget

    def __init__(self, ac_name, facebook_style=False, view=autocomplete,
                 min_chars = 1, token_limit = None, hint_text = None, no_result_text = None, 
                 searching_text = None, on_result=None, on_select_token=None,
                 on_delete_token=None, force_selection=True, **kwargs):
        self.ac_name = ac_name
        self.view = view
        widget = self.widget(ac_name=ac_name, force_selection=force_selection,
                             view=view, facebook_style=facebook_style, min_chars=min_chars, 
                             token_limit=token_limit, hint_text=hint_text, no_result_text=no_result_text,
                             searching_text=searching_text, on_result=on_result,
                             on_select_token=on_select_token, on_delete_token=on_delete_token)
        forms.Field.__init__(self, widget=widget, **kwargs)

    def _get_queryset(self):
        return self.view.settings[self.ac_name][0].objects.all()
    queryset = property(_get_queryset, forms.ModelChoiceField._set_queryset)

    @property
    def to_field_name(self):
        return self.view.settings[self.ac_name][3]


class AutocompleteModelChoiceField(forms.ModelChoiceField):
    """
    A ModelChoiceField which uses an autocomplete widget, instead of an html
    select tag, to render its choices.
    """
    widget = AutocompleteWidget

    def __init__(self, ac_name, facebook_style=False, view=autocomplete,
                 min_chars = 1, hint_text = None, no_result_text = None, searching_text = None,
                 on_result=None, on_select_token=None,
                 on_delete_token=None, force_selection=True, **kwargs):
        self.ac_name = ac_name
        self.view = view
        widget = self.widget(ac_name=ac_name, force_selection=force_selection,
                             view=view, facebook_style=facebook_style, min_chars=min_chars, 
                             hint_text=hint_text, no_result_text=no_result_text,
                             searching_text=searching_text, on_result=on_result, 
                             on_select_token=on_select_token, on_delete_token=on_delete_token)
        forms.Field.__init__(self, widget=widget, **kwargs)

    def _get_queryset(self):
        return self.view.settings[self.ac_name][0].objects.all()
    queryset = property(_get_queryset, forms.ModelChoiceField._set_queryset)

    @property
    def to_field_name(self):
        return self.view.settings[self.ac_name][3]
