from django.forms import widgets, fields
from django.forms.widgets import Input, SelectMultiple, Select, TextInput
from django.utils.safestring import mark_safe
from django.core.urlresolvers import reverse
from django.utils.encoding import force_unicode, smart_unicode
from django.forms.util import flatatt
from django.utils import simplejson, datetime_safe, formats
from django.utils.translation import ugettext as _
from django.contrib.contenttypes.models import ContentType
from django.db.models import get_model
from django.core.urlresolvers import get_callable

from jumbo.apps.autocomplete.views import autocomplete
from jumbo.middleware.thread_local import get_request

HINT_TEXT = _("Type in a search term")
NO_RESULT_TEXT = _("No results")
SEARCHING_TEXT = _("Searching...")

class CommonAutocomplete(object):
    def __init__(self, model=None, url=None, label=None, view=None, ac_name=None,
                 force_selection=True, facebook_style=False, min_chars=1,
                 token_limit=None, hint_text=None, no_result_text=None, searching_text=None,
                 on_result=None, on_select_token=None, on_delete_token=None, multiple=False):
        self.force_selection = bool(force_selection)
        self.facebook_style = facebook_style
        self.min_chars = min_chars
        self.token_limit = token_limit
        self.hint_text = hint_text or HINT_TEXT
        self.no_result_text = no_result_text or NO_RESULT_TEXT
        self.searching_text = searching_text or SEARCHING_TEXT
        self.url = url
        self.model = model
        self.view = view
        self.ac_name = ac_name
        self.label = label
        self.on_result = on_result
        self.on_select_token = on_select_token
        self.on_delete_token = on_delete_token
        self.multiple = multiple


    def get_media(self):
        if self.facebook_style:
            return widgets.Media(css={'all': ("admin/autocomplete/css/token-input-facebook.css",)},
                                 js=('admin/autocomplete/js/jquery.tokeninput.js',))
        else:
            return widgets.Media(css={'all': ("admin/autocomplete/css/token-input.css",)},
                                 js=('admin/autocomplete/js/jquery.tokeninput.js',))

    def get_prepopulated_values(self, value):
        model, fields, limit, key, label, auth, views, force_selection = self.view.settings[self.ac_name]

        if not self.force_selection:
            if type(value) == type([]):
                values = [{"id":v, "name": v} for v in value]
            else:
                values = [{"id":value, "name": value}]
        else:
            if type(value) == type([]):
                qs = model.objects.filter(pk__in=value)
            else:
                if type(value) == type(1):
                    qs = model.objects.filter(pk=value)
                elif type(value) == type(u'') or type(value) == type(''):
                    if value.isdigit():
                        qs = model.objects.filter(pk=value)
                    else:
                        qs = []
                else:
                    qs = []

            values = [{"id":getattr(obj, 'pk'), "name":label(obj)} for obj in qs]

        return simplejson.dumps(values)


    def render(self, name, value, attrs=None, choices=()):
        # Aggiuno il piu accanto alla multi select autocomplete
        model, fields, limit, key, label, auth, views, force_selection = self.view.settings[self.ac_name]
        add_href = None

        if model:
            app_label = model._meta.app_label
            model_name = model._meta.object_name.lower()
            add_href = """<a onclick="return showAddAnotherPopup(this);" id="add_id_%(model_name)s" class="add-another add-another-autocomplete" href="/admin/%(app_label)s/%(model_name)s/add/">
                            <img height="10" width="10" alt="Add Another" src="/media/img/admin/icon_addlink.gif">
                        </a>""" % {'model_name':model_name, 'app_label': app_label}

        # scelgo il tipo di select da renderizzare in base se i valori da selezionare
        # sono multipli o singoli
        if self.multiple:
            html = '<select id="id_%(name)s" name="%(name)s" multiple="multiple" style="display: none;"></select>'
        else:
            html = '<select id="id_%(name)s" name="%(name)s" style="display: none;"></select>'
        if add_href:
            html += add_href
        html = html % {'name':name}

        # Se gli viene passato un url non uso piu' la gestione automatica, ma chiamo
        # direttamente l'url passato
        if self.url:
            url = self.url
        else:
            url = "/autocomplete/%s/" % self.ac_name

        # Collego al widget il javascript che gli aggiungera' la funzione
        # dell'autocompletamento. In base ai parametri passati dall'utente
        # aggiungo o meno i settaggi dell'autocomplete
        html += u"""<script type="text/javascript">
                jQuery(document).ready(function($) {
                  $("#id_%(name)s").tokenInput("%(url)s", {
                        %%(classes)s
                        %%(populate)s
                        %%(token_limit)s
                        %%(on_result)s
                        %%(on_select_token)s
                        %%(on_delete_token)s
                        %%(hint_text)s,
                        %%(no_result_text)s,
                        %%(searching_text)s,
                        %%(min_chars)s,
                        %%(force_selection)s
                    });
                });
                </script>""" % {'name': name, 'url':url,}

        # Default Style
        classes = ""
        if self.facebook_style:
            # Facebook Style
            classes = """classes: {
                                tokenList: "token-input-list-facebook",
                                token: "token-input-token-facebook",
                                tokenDelete: "token-input-delete-token-facebook",
                                selectedToken: "token-input-selected-token-facebook",
                                highlightedToken: "token-input-highlighted-token-facebook",
                                dropdown: "token-input-dropdown-facebook",
                                dropdownItem: "token-input-dropdown-item-facebook",
                                dropdownItem2: "token-input-dropdown-item2-facebook",
                                selectedDropdownItem: "token-input-selected-dropdown-item-facebook",
                                inputToken: "token-input-input-token-facebook"
                        },"""

        populate = ""
        if value:
            value = self.get_prepopulated_values(value)
            populate = "'prePopulate':%s," % value

        hint_text = "'hintText': '%s'" % _(self.hint_text)
        no_result_text  = "'noResultsText': '%s'" % _(self.no_result_text)
        searching_text  = "'searchingText': '%s'" % _(self.searching_text)
        min_chars  = "'minChars': %s" % self.min_chars

        token_limit = ''
        if self.token_limit:
            token_limit = "'tokenLimit': %s," % self.token_limit

        on_result = ''
        if self.on_result:
            on_result = "'onResult': %s," % self.on_result

        on_select_token = ''
        if self.on_select_token:
            on_select_token = "'onSelectToken': %s," % self.on_select_token

        on_delete_token = ''
        if self.on_delete_token:
            on_delete_token = "'onDeleteToken': %s," % self.on_delete_token

        force_selection = "'forceSelection':false"
        if self.force_selection:
            force_selection = "'forceSelection':true"

        html = html % {'classes' : classes, 'populate': populate, 'hint_text':hint_text,
                       'no_result_text' : no_result_text, 'searching_text' : searching_text,
                       'min_chars': min_chars, 'token_limit': token_limit, 'on_result': on_result,
                       'on_select_token': on_select_token, 'on_delete_token': on_delete_token,
                       'force_selection': force_selection}
        return mark_safe(html)


class AutocompleteWidget(Select):
    def _media(self):
        return self.common.get_media()
    media = property(_media)

    def __init__(self, ac_name, force_selection=True, view=autocomplete,
                 facebook_style=False, min_chars=1, hint_text=None,
                 no_result_text=None, searching_text=None, on_result=None,
                 on_select_token=None, on_delete_token=None):
        super(AutocompleteWidget, self).__init__()
        self.common = CommonAutocomplete(ac_name=ac_name, view=view,
                         force_selection=force_selection,
                         facebook_style=facebook_style, min_chars=min_chars,
                         token_limit=1, hint_text=hint_text,
                         no_result_text=no_result_text, searching_text=searching_text,
                         on_result=on_result, on_select_token=on_select_token,
                         on_delete_token=on_delete_token, multiple=False)

    def render(self, name, value, attrs=None, choices=()):
        return self.common.render(name, value, attrs, choices)


class AutocompleteMultipleWidget(SelectMultiple):
    def _media(self):
        return self.common.get_media()
    media = property(_media)

    def __init__(self, ac_name, force_selection=True,
                 view=autocomplete, facebook_style=False, min_chars=1, token_limit=None,
                 hint_text=None, no_result_text=None, searching_text=None, on_result=None,
                 on_select_token=None, on_delete_token=None):
        super(AutocompleteMultipleWidget, self).__init__()
        self.common = CommonAutocomplete(ac_name=ac_name, view=view,
                         force_selection=force_selection,
                         facebook_style=facebook_style, min_chars=min_chars,
                         token_limit=token_limit, hint_text=hint_text,
                         no_result_text=no_result_text, searching_text=searching_text,
                         on_result=on_result, on_select_token=on_select_token,
                         on_delete_token=on_delete_token, multiple=True)

    def render(self, name, value, attrs=None, choices=()):
        return self.common.render(name, value, attrs, choices)


class AutocompleteInlineCharFieldWidget(TextInput):
    """
        AUTOCOMPLETE PER INLINE - INPUT TEXT
        Widget da utilizzare esclusivamente negli inline. La renderizzazione di questo
        widget non attacca la parte relativa al javascript che dovra' essere gestita
        altrove. Questo e' dovuto al funzionamento "live" degli inline di django.
    """
    def _media(self):
        return self.common.get_media()
    media = property(_media)

    def __init__(self, ac_name, view=autocomplete):
        super(AutocompleteInlineCharFieldWidget, self).__init__()
        self.common = CommonAutocomplete(ac_name=ac_name, view=view)

    def render(self, name, value, attrs=None):
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = self.common.get_prepopulated_values(value)
        return mark_safe(u'<input%s />' % flatatt(final_attrs))

class AutocompleteInlineGFKCharFieldWidget(TextInput):
    """
        AUTOCOMPLETE PER INLINE - INPUT TEXT SU UNA GENERIC FOREIGN KEY
        Widget da utilizzare esclusivamente negli inline.
        L'output combina i due valori (Content type e object id)
        sui quali e' applicata la generic foreign key
        Il widget si aspetta che value sia costruito con la sintassi contentType-object_id (es: 24-1)
    """
    def _media(self):
        return self.common.get_media()
    media = property(_media)

    def __init__(self, ac_name, view=autocomplete):
        super(AutocompleteInlineGFKCharFieldWidget, self).__init__()
        self.common = CommonAutocomplete(ac_name=ac_name, view=view)

    def get_prepopulated_values(self, value):

        list = value.split('-')
        ct = ContentType.objects.get(pk=list[0])
        obj = get_model(ct.app_label, ct.model)
        obj_name = obj.objects.get(pk=list[1])
        label = '%s | %s' %(ct.name, obj_name)

        values = [{"id":value, "name": label}]

        return simplejson.dumps(values)

    def render(self, name, value, attrs=None):
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = self.get_prepopulated_values(value)
        return mark_safe(u'<input%s />' % flatatt(final_attrs))


#class AutocompleteWidget(Select):
#    """
#        AUTOCOMPLETE MANUALE - SELECT
#        Widget da usare quando il valore che puo' selezionare l'utente e' uno solo.
#        Questo widget dovra' essere gestito manualmente dall'utente che gli dovra'
#        passare sia il modello che l'url da chiamare.
#    """
#    def _media(self):
#        return self.common.get_media()
#    media = property(_media)

#    def __init__(self, model, url, label = lambda obj: smart_unicode(obj), force_selection=True,
#                 reverse_label=True, facebook_style=False, min_chars=1,
#                 hint_text=None, no_result_text=None, searching_text=None, on_result=None,
#                 on_select_token=None):
#        super(AutocompleteWidget, self).__init__()
#        self.common = CommonAutocomplete(model=model, url=url, label=label,
#                                           force_selection=force_selection, reverse_label=reverse_label,
#                                           facebook_style=facebook_style, min_chars=min_chars,
#                                           token_limit=1, hint_text=hint_text,
#                                           no_result_text=no_result_text, searching_text=searching_text,
#                                           on_result=on_result, on_select_token=on_select_token, multiple=False)

#    def render(self, name, value, attrs=None, choices=()):
#        return self.common.render(name, value, attrs, choices)


#class AutocompleteMultipleWidget(SelectMultiple):
#    """
#        AUTOCOMPLETE MANUALE - MUTLI SELECT
#        Widget da usare quando l'utente puo' selezionare piu' valori.
#        Questo widget dovra' essere gestito manualmente dall'utente che gli dovra'
#        passare sia il modello che l'url da chiamare.
#    """
#    def _media(self):
#        return self.common.get_media()
#    media = property(_media)

#    def __init__(self, model, url, label = lambda obj: smart_unicode(obj), force_selection=True,
#                 reverse_label=True, facebook_style=False, min_chars=1, token_limit=None,
#                 hint_text=None, no_result_text=None, searching_text=None, on_result=None,
#                 on_select_token=None):
#        super(AutocompleteSelectMultipleWidget, self).__init__()
#        self.common = CommonAutocomplete(model=model, url=url, label=label,
#                                           force_selection=force_selection, reverse_label=reverse_label,
#                                           facebook_style=facebook_style, min_chars=min_chars,
#                                           token_limit=token_limit, hint_text=hint_text,
#                                           no_result_text=no_result_text, searching_text=searching_text,
#                                           on_result=on_result, on_select_token=on_select_token, multiple=True)

#    def render(self, name, value, attrs=None, choices=()):
#        return self.common.render(name, value, attrs, choices)
