# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from jumbo.db.models import OrderedModel, StatusModel, DateModel, UserModel
from jumbo.apps.i18n.models import I18NModel, TranslationModel
from jumbo.apps.template.models import Template

OPEN_CHOICES = (
    ('1', _('Open in same window')),
    ('2', _('Open in new window')),
    ('3', _('Open in popup')),
    ('4', _('Call javascript')),
    ('5', _('Don\'t open')),
)

class CategoryManager(models.Manager):
    def default(self):
        return self.get_query_set().get(default=True)

class Category(DateModel, UserModel, StatusModel, OrderedModel):
    objects = CategoryManager()

    ## campi modello
    name = models.CharField(
        unique=True,
        max_length=25,
        verbose_name=_('category name'),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('Default category'),
        help_text=_('Is the default one?'),
    )

    def __unicode__(self):
        return u"%s" % self.name

    class Meta:
        verbose_name = _('Menu Category')
        verbose_name_plural = _('Menu Categories')

    def save(self, *args, **kwargs):
        super(Category, self).save(*args, **kwargs)
        if self.default:
            for sc in Category.objects.exclude(pk=self.id):
                sc.default = False
                sc.save(*args, **kwargs)
        try:
            sc = Category.objects.get(default=True)
        except models.ObjectDoesNotExist:
            self.default = True
            super(Category, self).save(*args, **kwargs)

class Menu(OrderedModel, StatusModel, I18NModel):
    parent = models.ForeignKey(
        'self',
        null=True, blank=True,
        related_name='children',
        db_index=True,
        verbose_name=_('parent menu'),
        help_text=_('parent menu'),
    )
    name = models.CharField(
        max_length=50,
        db_index=True,
        verbose_name=_('name for this node'),
        help_text=_('name for this node'),
    )
    category = models.ForeignKey(
        Category,
        null=True, blank=True,
        related_name='menu',
        verbose_name=_('category menu'),
    )
    url = models.CharField(
        max_length=256,
        blank=True,
        null=True,
        db_index=True,
        verbose_name=_('url'),
        help_text=_('url'),
    )
    open_choices = models.CharField(
        max_length=5,
        default='1',
        choices=OPEN_CHOICES,
        db_index=True,
        verbose_name=_('open in'),
        help_text=_('open in'),
    )
    caption = models.CharField(
        max_length=50,
        blank=True,
        null=True,
        verbose_name=_('Application Name'),
    )
    slug = models.SlugField(
        blank=True,
        db_index=True,
        verbose_name=_('slug'),
        help_text=_('slug'),
    )
    template = models.ForeignKey(
        Template,
        null=True, blank=True,
        db_index=True,
        related_name='menu',
        verbose_name=_('template to render'),
        help_text=_('template to render'),
    )
    permission = models.CharField(
        max_length=100,
        blank=True,
        null=True,
        verbose_name=_('permission'),
        help_text=_('Permission user must have to view this menu'),
    )
    def _get_name_parent(self, obj):
        p_list = []
        if obj.parent_id:
            p = obj.parent
            p_list.append(u' / ')
            p_list.append(p.name)
            more = self._get_name_parent(p)
            p_list.extend(more)
        if obj == self and p_list:
            p_list.reverse()
        return p_list
    
    def _get_full_name(self):
        return self.__unicode__()
    full_name = property(_get_full_name)

    def _has_children(self):
        return self.children.count() > 0
    has_children = property(_has_children)

    def __unicode__(self):
        p_list = self._get_name_parent(self)
        p_list.append(self.name)
        return u''.join(p_list)

    def save(self):
        recalculate_order = False
        self.slug = slugify(self.name)
        if not self.category and self.parent:
            self.category = self.parent.category
        if not self.caption:
            self.caption = self.name
        super(Menu, self).save()

    class Meta:
        verbose_name = _('Menu')
        verbose_name_plural = _('Menu')

class MenuTranslation(TranslationModel):
    menu = models.ForeignKey(
        Menu,
        related_name='translations',
        verbose_name=_('menu')
    )
    caption = models.CharField(
        max_length=50,
        verbose_name=_('translated caption'),
    )
    name = models.CharField(
        max_length=50,
        verbose_name=_('translated name'),
        blank = True, null = True
    )

    class Meta:
        verbose_name = _('menu translation')
        verbose_name_plural = _('menu translation')
        unique_together = (('language', 'menu',),)

    def __unicode__(self):
        return u'%s %s' % (self.menu, self.caption)

MENU_ICON_TYPE_CHOICES = (
    ('normal', _('Normal state')),
    ('active', _('Active')),
    ('on-mouse-over', _('On mouse over')),
    ('on-mouse-out', _('On mouse out')),
    ('custom', _('Custom')),
)

class MenuIcon(models.Model):
    menu = models.ForeignKey(
        Menu,
        related_name='icons',
        verbose_name=_('menu')
    )
    icon = models.ImageField(
        upload_to='menu/',
        max_length=255,
        verbose_name=_('icon image'),
    )
    type = models.CharField(
        max_length=20,
        choices=MENU_ICON_TYPE_CHOICES,
        verbose_name=_('Icon type'),
    )
    custom = models.CharField(
        max_length=20,
        blank=True,
        null=True,
        verbose_name=_('Custom icon type'),
    )

    class Meta:
        verbose_name = _('menu icon')
        verbose_name_plural = _('menu icons')
        unique_together = (('menu', 'type', 'custom',),)

    def __unicode__(self):
        if self.type == 'custom':
            return u'%s %s' % (self.menu, self.custom)
        else:
            return u'%s %s' % (self.menu, self.type)

    def save(self, *args, **kwargs):
        if self.type != MENU_ICON_TYPE_CHOICES[-1][0] and self.custom:
            self.custom = None
        super(MenuIcon, self).save(*args, **kwargs)

