# -*- coding: utf-8 -*-
# get_cache_backend()
# backend
# add_template_to_cache(instance, **kwargs)
# remove_cached_template(instance, **kwargs)
# are based on django-dbtemplates
#
# Copyright (c) 2007-2008, Jannis Leidel
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of the author nor the names of other
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import datetime

from django.db import models
from django.conf import settings
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from jumbo.apps.i18n.models import TranslationModel, I18NModel
from jumbo.middleware.thread_local import get_request
#from jumbo.db.models import HTMLField

class Template(I18NModel):
    name = models.CharField(
        max_length=50,
        verbose_name=_('name'),
        help_text=_('Name of this template'),
    )
    active = models.BooleanField(
        default=True,
        verbose_name=_('active'),
        help_text=_('If this template can be used.'),
    )
    slug = models.SlugField(
        blank=True,
        db_index=True,
        verbose_name=_('slug'),
        help_text=_('slug'),
    )
    description = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('description'),
        help_text=_('Purpose of this template.'),
    )
    default = models.TextField(
        blank=True,
        null=True,
        verbose_name=_('Default Text'),
        help_text=_('Default text, if no translation is provided.'),
    )
    meta_keywords = models.CharField(
        blank=True,
        null=True,
        max_length = 255,
        verbose_name = _("Meta Keywords")
    )
    meta_description = models.CharField(
        blank=True,
        null=True,
        max_length = 255,
        verbose_name = _("Meta Description")
    )

    def _text(self):
        return self.default
    text = property(_text)

    def _title(self):
        return self.description
    title = property(_title)

    def __unicode__(self):
        return self.name

    def save(self):
        self.slug = slugify(self.name)
        super(Template, self).save()

    class Meta:
        verbose_name = _('template')
        verbose_name_plural = _('templates')
        ordering = ('name',)

class TemplateTranslation(TranslationModel):
    template = models.ForeignKey(
        Template,
        related_name='translations',
        verbose_name=_('template translation'),
        help_text=_('template translation'),
    )
    title = models.CharField(
        max_length = 255,
        blank=True,
        null=True,
        verbose_name = _("Title")
    )
    text = models.TextField(
        blank=True,
        null=True,
        verbose_name=_('translated text'),
        help_text=_('translated text'),
    )
    meta_keywords = models.CharField(
        blank=True,
        null=True,
        max_length = 255,
        verbose_name = _("Meta Keywords")
    )
    meta_description = models.CharField(
        max_length = 255,
        blank=True,
        null=True,
        verbose_name = _("Meta Description")
    )

    def __unicode__(self):
        try:
            parent = self.template_set
            return u'%(parent)s (%(lang)s)' % {'lang': self.language.name, 'parent': parent}
        except:
            pass

        if len(self.text) > 59:
            return '%(lang)s: %(text)s...' % {'text':self.text[0:60], 'lang': self.language.name}
        else:
            return u'%(lang)s: %(text)s' % {'text':self.text, 'lang': self.language.name}
    class Meta:
        verbose_name = _('template translation')
        verbose_name_plural = _('template translation')

def get_cache_backend():
    path = getattr(settings, 'JUMBO_TEMPLATE_CACHE_BACKEND', False)
    if path:
        i = path.rfind('.')
        module, attr = path[:i], path[i+1:]
        try:
            mod = __import__(module, {}, {}, [attr])
        except ImportError, e:
            raise ImproperlyConfigured, 'Error importing template cache backend %s: "%s"' % (module, e)
        try:
            cls = getattr(mod, attr)
        except AttributeError:
            raise ImproperlyConfigured, 'Module "%s" does not define a "%s" cache backend' % (module, attr)
        return cls()
    return False

backend = get_cache_backend()

def add_template_to_cache(instance, **kwargs):
    """
    Called via Django's signals to cache the templates, if the template
    in the database was added or changed.
    """
    backend.save(instance.name, instance.content)

def remove_cached_template(instance, **kwargs):
    """
    Called via Django's signals to remove cached templates, if the template
    in the database was changed or deleted.
    """
    backend.remove(instance.name)

if backend:
    signals.post_save.connect(remove_cached_template, sender=Template)
    signals.post_save.connect(add_template_to_cache, sender=Template)
    signals.pre_delete.connect(remove_cached_template, sender=Template)

